from datetime import datetime, timedelta

from pydantic import BaseModel, Field


class Album(BaseModel):
    caption: str = Field("", alias="Caption")
    category: str = Field("", alias="Category")
    country: str = Field("", alias="Country")
    created_at: datetime = Field(alias="CreatedAt")
    created_by: str = Field("", alias="CreatedBy")
    day: int = Field(alias="Day")
    deleted_at: datetime | None = Field(alias="DeletedAt")
    description: str = Field("", alias="Description")
    favorite: bool = Field(alias="Favorite")
    filter: str = Field("", alias="Filter")
    id: int = Field(alias="ID")
    location: str = Field("", alias="Location")
    month: int = Field(alias="Month")
    notes: str = Field("", alias="Notes")
    order: str = Field("", alias="Order")
    parent_uid: str = Field("", alias="ParentUID")
    path: str = Field("", alias="Path")
    private: bool = Field(alias="Private")
    published_at: datetime | None = Field(None, alias="PublishedAt")
    slug: str = Field("", alias="Slug")
    state: str = Field("", alias="State")
    template: str = Field("", alias="Template")
    thumb: str = Field("", alias="Thumb")
    thumb_src: str = Field("", alias="ThumbSrc")
    title: str = Field("", alias="Title")
    type: str = Field("", alias="Type")
    uid: str = Field("", alias="UID")
    updated_at: datetime = Field("", alias="UpdatedAt")
    year: int = Field(alias="Year")


class Camera(BaseModel):
    description: str | None = Field(None, alias="Description")
    id: int = Field(alias="ID")
    make: str = Field(alias="Make")
    model: str = Field(alias="Model")
    name: str = Field(alias="Name")
    slug: str = Field(alias="Slug")
    type: str | None = Field(None, alias="Type")


class Country(BaseModel):
    description: str = Field(alias="Description")
    id: str = Field(alias="ID")
    name: str = Field(alias="Name")
    notes: str = Field(alias="Notes")
    slug: str = Field(alias="Slug")


class Lens(BaseModel):
    description: str | None = Field(None, alias="Description")
    id: int = Field(alias="ID")
    make: str = Field(alias="Make")
    model: str = Field(alias="Model")
    name: str = Field(alias="Name")
    notes: str | None = Field(None, alias="Notes")
    slug: str = Field(alias="Slug")
    type: str = Field(alias="Type")


class Person(BaseModel):
    alias: str = Field(alias="Alias")
    favorite: bool = Field(alias="Favorite")
    hidden: bool = Field(alias="Hidden")
    name: str = Field(alias="Name")
    uid: str = Field(alias="UID")


class Link(BaseModel):
    uid: str = Field(alias="UID")
    share_uid: str = Field(alias="ShareUID")
    slug: str = Field(alias="Slug")
    token: str = Field(alias="Token")
    expires: int = Field(alias="Expires")
    views: int = Field(alias="Views")
    max_views: int = Field(alias="MaxViews")
    verify_password: bool = Field(alias="VerifyPassword")
    comment: str = Field("", alias="Comment")
    perm: int | None = Field(None, alias="Perm")
    created_by: str = Field(alias="CreatedBy")
    created_at: datetime = Field(alias="CreatedAt")
    modified_at: datetime = Field(alias="ModifiedAt")


class Label(BaseModel):
    id: int = Field(alias="ID")
    uid: str = Field(alias="UID")
    slug: str = Field(alias="Slug")
    custom_slug: str = Field(alias="CustomSlug")
    name: str = Field(alias="Name")
    priority: int = Field(alias="Priority")
    favorite: bool = Field(alias="Favorite")
    description: str = Field(alias="Description")
    notes: str = Field(alias="Notes")
    photo_count: int = Field(alias="PhotoCount")
    thumb: str = Field(alias="Thumb")
    thumb_src: str = Field(alias="ThumbSrc")
    created_at: datetime = Field(alias="CreatedAt")
    updated_at: datetime = Field(alias="UpdatedAt")
    published_at: datetime = Field(alias="PublishedAt")
    deleted_at: datetime = Field(alias="DeletedAt")


class Marker(BaseModel):
    uid: str = Field(alias="UID")
    file_uid: str = Field(alias="FileUID")
    type: str = Field(alias="Type")
    src: str = Field(alias="Src")
    name: str = Field(alias="Name")
    review: bool = Field(alias="Review")
    invalid: bool = Field(alias="Invalid")
    subj_uid: str = Field(alias="SubjUID")
    subj_src: str = Field(alias="SubjSrc")
    face_id: str = Field(alias="FaceID")
    face_dist: float = Field(alias="FaceDist")
    x: float = Field(alias="X")
    y: float = Field(alias="Y")
    w: float = Field(alias="W")
    h: float = Field(alias="H")
    q: int = Field(alias="Q")
    size: int = Field(alias="Size")
    score: int = Field(alias="Score")
    thumb: str = Field(alias="Thumb")
    matched_at: datetime = Field(alias="MatchedAt")


class File(BaseModel):
    photo_uid: str = Field(alias="PhotoUID")
    taken_at: datetime = Field(alias="TakenAt")
    time_index: str = Field(alias="TimeIndex")
    media_id: str = Field(alias="MediaID")
    media_utc: int = Field(alias="MediaUTC")
    instance_id: str = Field(alias="InstanceID")
    uid: str = Field(alias="UID")
    name: str = Field(alias="Name")
    root: str = Field(alias="Root")
    original_name: str = Field(alias="OriginalName")
    hash: str = Field(alias="Hash")
    size: int = Field(alias="Size")
    codec: str = Field(alias="Codec")
    file_type: str = Field(alias="FileType")
    media_type: str = Field(alias="MediaType")
    mime: str = Field(alias="Mime")
    primary: bool = Field(alias="Primary")
    sidecar: bool = Field(alias="Sidecar")
    missing: bool = Field(alias="Missing")
    portrait: bool = Field(alias="Portrait")
    video: bool = Field(alias="Video")
    duration: timedelta = Field(alias="Duration")
    fps: float = Field(alias="FPS")
    frames: int = Field(alias="Frames")
    width: int = Field(alias="Width")
    height: int = Field(alias="Height")
    orientation: int = Field(alias="Orientation")
    orientation_src: str = Field(alias="OrientationSrc")
    projection: str = Field(alias="Projection")
    aspect_ratio: float = Field(alias="AspectRatio")
    hdr: bool = Field(alias="HDR")
    watermark: bool = Field(alias="Watermark")
    color_profile: str = Field(alias="ColorProfile")
    main_color: str = Field(alias="MainColor")
    colors: str = Field(alias="Colors")
    luminance: str = Field(alias="Luminance")
    diff: int = Field(alias="Diff")
    chroma: int = Field(alias="Chroma")
    software: str = Field(alias="Software")
    error: str = Field(alias="Error")
    mod_time: str = Field(alias="ModTime")
    created_at: datetime = Field(alias="CreatedAt")
    created_in: int = Field(alias="CreatedIn")
    updated_at: datetime = Field(alias="UpdatedAt")
    updated_in: int = Field(alias="UpdatedIn")
    published_at: datetime = Field(alias="PublishedAt")
    deleted_at: datetime = Field(alias="DeletedAt")


class Details(BaseModel):
    keywords: str = Field(alias="Keywords")
    keywords_src: str = Field(alias="KeywordsSrc")
    notes: str = Field(alias="Notes")
    notes_src: str = Field(alias="NotesSrc")
    subject: str = Field(alias="Subject")
    subject_src: str = Field(alias="SubjectSrc")
    artist: str = Field(alias="Artist")
    artist_src: str = Field(alias="ArtistSrc")
    copyright: str = Field(alias="Copyright")
    copyright_src: str = Field(alias="CopyrightSrc")
    license: str = Field(alias="License")
    license_src: str = Field(alias="LicenseSrc")
    software: str = Field(alias="Software")
    software_src: str = Field(alias="SoftwareSrc")


class Place(BaseModel):
    place_id: str = Field(alias="PlaceID")
    label: str = Field(alias="Label")
    district: str = Field(alias="District")
    city: str = Field(alias="City")
    state: str = Field(alias="State")
    country: str = Field(alias="Country")
    keywords: str = Field(alias="Keywords")
    favorite: bool = Field(alias="Favorite")
    photo_count: int = Field(alias="PhotoCount")
    created_at: datetime = Field(alias="CreatedAt")
    updated_at: datetime = Field(alias="UpdatedAt")


class Cell(BaseModel):
    id: str = Field(alias="ID")
    name: str = Field(alias="Name")
    street: str = Field(alias="Street")
    postcode: str = Field(alias="Postcode")
    category: str = Field(alias="Category")
    place: Place = Field(alias="Place")
    created_at: datetime = Field(alias="CreatedAt")
    updated_at: datetime = Field(alias="UpdatedAt")


class Photo(BaseModel):
    document_id: str = Field("", alias="DocumentID")
    taken_at: datetime = Field(alias="TakenAt")
    taken_at_local: datetime = Field(alias="TakenAtLocal")
    taken_src: str = Field("", alias="TakenSrc")
    uid: str = Field("", alias="UID")
    type: str = Field("", alias="Type")
    type_src: str = Field("", alias="TypeSrc")
    title: str = Field("", alias="Title")
    title_src: str = Field("", alias="TitleSrc")
    description: str = Field("", alias="Description")
    description_src: str = Field("", alias="DescriptionSrc")
    path: str = Field("", alias="Path")
    name: str = Field("", alias="Name")
    original_name: str = Field("", alias="OriginalName")
    stack: int = Field(alias="Stack")
    favorite: bool = Field(alias="Favorite")
    private: bool = Field(alias="Private")
    scan: bool = Field(alias="Scan")
    panorama: bool = Field(alias="Panorama")
    time_zone: str = Field("", alias="TimeZone")
    place_id: str = Field("", alias="PlaceID")
    place_src: str = Field("", alias="PlaceSrc")
    cell_id: str = Field("", alias="CellID")
    cell_accuracy: int | None = Field(None, alias="CellAccuracy")
    altitude: int | None = Field(None, alias="Altitude")
    lat: float = Field(alias="Lat")
    lng: float = Field(alias="Lng")
    country: str = Field("", alias="Country")
    year: int = Field(alias="Year")
    month: int = Field(alias="Month")
    day: int = Field(alias="Day")
    iso: int = Field(alias="Iso")
    exposure: str = Field("", alias="Exposure")
    f_number: float = Field(alias="FNumber")
    focal_length: int = Field(alias="FocalLength")
    quality: int = Field(alias="Quality")
    faces: int | None = Field(None, alias="Faces")
    resolution: int = Field(alias="Resolution")
    duration: timedelta | None = Field(None, alias="Duration")
    color: int = Field(alias="Color")
    camera_id: int = Field(alias="CameraID")
    camera_src: str = Field("", alias="CameraSrc")
    lens_id: int = Field(alias="LensID")
    details: Details = Field(alias="Details")
    camera: Camera = Field(alias="Camera")
    lens: Lens = Field(alias="Lens")
    cell: Cell = Field(alias="Cell")
    place: Place = Field(alias="Place")
    albums: list[Album] = Field([], alias="Albums")
    created_by: str = Field("", alias="CreatedBy")
    created_at: datetime = Field(alias="CreatedAt")
    updated_at: datetime = Field(alias="UpdatedAt")
    edited_at: datetime | None = Field(None, alias="EditedAt")
    published_at: datetime | None = Field(None, alias="PublishedAt")
    checked_at: datetime = Field(alias="CheckedAt")
    estimated_at: datetime | None = Field(None, alias="EstimatedAt")
    deleted_at: datetime | None = Field(None, alias="DeletedAt")


class Error(BaseModel):
    id: int = Field(alias="ID")
    time: datetime = Field(alias="Time")
    level: str = Field(alias="Level")
    message: str = Field(alias="Message")


class Face(BaseModel):
    id: str = Field(alias="ID")
    src: str = Field(alias="Src")
    kind: int = Field(alias="Kind")
    hidden: bool = Field(alias="Hidden")
    subj_uid: str = Field(alias="SubjUID")
    samples: int = Field(alias="Samples")
    sample_radius: float = Field(alias="SampleRadius")
    collisions: int = Field(alias="Collisions")
    collision_radius: float = Field(alias="CollisionRadius")
    matched_at: datetime = Field(alias="MatchedAt")
    created_at: datetime = Field(alias="CreatedAt")
    updated_at: datetime = Field(alias="UpdatedAt")
